/* 
 * Kitty Currier, March 2014
 * MAT 259 project 4
 *
 * Data reflect changes in library items' bar codes;
 * each jump to a larger circle represents date at
 * which an item was checked out under a new bar code.
 *
 * Required libraries:
 * G4P by Peter Lager (http://www.lagers.org.uk/g4p/index.html)
 * PeasyCam by Jonathan Feinberg (http://mrfeinberg.com/peasycam/)
 */
 
import g4p_controls.*;
import peasy.*;
 
void setup() {
  size(windowW, windowH, OPENGL);
//  if (frame != null) {
//    frame.setResizable(true);
//  }
  smooth();
  colorMode(HSB, 100);
  plainColor = color(0, 0, 50, 20); //color for drawing un-highlighted trajectories
    
  f72 = createFont("Georgia", 72);
  f26 = createFont("Georgia", 26);
  h14 = createFont("Helvetica", 14);
  h16 = createFont("Helvetica", 16);
  h72 = createFont("Helvetica", 72);
  h26 = createFont("Helvetica", 26);
  
  cam = new PeasyCam(this, 0, -450, 0, 3500);
  cam.setMinimumDistance(50);
  cam.setMaximumDistance(6800);
  cam.setWheelScale(1.8);
  cam.setRotations(radians(8), 0, 0); //(pitch, yaw, roll)
  
  //load data, shuffle & check how many rows to draw
  data = new Table(filepath, true);
  println("Loaded "+data.getRowCount()+" rows from table "+filepath);
  if(rowsToDraw <= 0) {
    rowsToDraw = data.getRowCount();
  } else if(rowsToDraw < data.getRowCount()){
    rowsToDraw += 1;
    println("rowsToDraw: "+rowsToDraw);
  } else if(rowsToDraw >= data.getRowCount()) {
    rowsToDraw = data.getRowCount(); 
  }
  
  //GUI elements
  //slider code must come after loading data
  cam.beginHUD();
  sdr1 = new GCustomSlider(this, 20, 60, 250, 50, null); // Constructor is `Parent applet', the x, y position, length, width
  sdr1.setShowDecor(false, true, true, true);
  sdr1.setNbrTicks(5);
  sdr1.setLimits(100, 1, data.getRowCount()-1);
  cbxLocCentral = new GCheckbox(this, 300, 55, 65, 20, "Central");
  cbxLocOther= new GCheckbox(this, 300, 75, 60, 20, "Other");
  cbxTypeBk = new GCheckbox(this, 380, 55, 60, 20, "Book");
  cbxTypeCd = new GCheckbox(this, 380, 75, 60, 20, "CD");
  cbxTypeDvd = new GCheckbox(this, 380, 95, 60, 20, "DVD");
  cbxTypeKit = new GCheckbox(this, 440, 55, 60, 20, "Kit");
  cbxTypeOther = new GCheckbox(this, 440, 75, 60, 20, "Other");
  cam.endHUD();
}


void draw(){
  background(0);
  lights();
  directionalLight(1, 0, 80, // Color
                   1, -1, 1); // The x-, y-, z-axis direction
  
  // draw axes
//  strokeWeight(10);
//  stroke(100,0,0);
//  line(0,xyAxisLength,0,-xyAxisLength); // y axis
//  stroke(0,100,0);
//  line(-xyAxisLength,0,xyAxisLength,0); // x axis
//  stroke(0,0,100);
//  line(0,0,0,0,0,zAxisLength); // z axis
//  fill(1,1,95);
//  drawCylinder(20, 10, zAxisLength); //alternate 3D cylinder axis (int sides, float r, float h)
  
  //draw axis labels
  noStroke();
  fill(0, 0, 60, 80);
  textFont(f72);
  float offset = 89;
  for(int i=0; i<months.length; i++) {
    drawArcText(months[i], xyAxisLength+100, 0, 0, 0, offset+30*i);
  }

  //draw year disks & labels
  noStroke();
  pushMatrix();
  textFont(h26);
  for(int i=0; i<8; i++) {
    fill(0, 0, 50, 60);
    ellipse(0, 0, rYearDisk, rYearDisk);
    String text = new String(str(i+2006));
    fill(0, 0, 99, 99);
    drawArcText(text, rYearDisk*0.2, 0, 0, 1, i*30+150);
    translate(0, 0, 360*zStep);
  }
  popMatrix();

  //draw each line/arc segment
  strokeWeight(2);
  for(int j = 1; j<rowsToDraw; j++) { //first row is a header
    applyFilter(data.getString(j, 6), data.getInt(j, 5), plainColor); //sets stroke color based on result  
    stroke(currentColor);
    for(int i = 0; i<5; i++) { //read first 5 cols
    int next = i+1;
      if((data.getInt(j, next) == -999) || (i == 4)) { //if the next value is -999 (no data), or reached col 4, do this
        drawLineArc(data.getInt(j, i), 2880, i); //2520 is the # of days between 2006 & 2013
        break;
      } else {
        drawLineArc(data.getInt(j,i), data.getInt(j,next), i);
      }   
    }
  }
  
    //draw UI background
  //use HUD to write stuff that is stationary in relation to the camera
  cam.beginHUD();
  pushStyle();
  noStroke();
  fill(1, 0, 100, 85);
  rect(0, 0, windowW, guiH);
  textFont(h16);
  textAlign(LEFT);
//  stroke(1,0,100, 99);
  text("No. of item paths to display", 20, 50);
  text(new String("Home Location"), 301, 5, 80, 50);
  text("Item Type", 381, 50);
  textFont(h72);
  textAlign(RIGHT);
  textSize(40);
  fill(1,0,0,100);
  text("bar code histories", windowW-20, 50);
  textFont(h14);
  fill(1, 0, 100, 85);
  text(description1, windowW-20, 90);
  text(description2, windowW-20, 110);
  text(description3, windowW-20, 130);
  textFont(h14);
  fill(1,0,30,100);
  text("Kitty Currier, March 2014", windowW-5, windowH-5);
  popStyle();
  cam.endHUD();

  setCamState(); //de-activates cam when mouse is over GUI
}
