
//check to see which items should be highlighted based on checkbox states
// toHighlight assignments:
// [ 0    1    2    3     4        5          6  ]
// {book, cd, dvd, kit, other, centralLoc, otherLoc}
void applyFilter(String type, int bool, color c) {
  currentColor = c;
    if(type.equals("bk") && toHighlight[0] == true) {
      currentColor = color(70, 50, 70, 60);
    } else if(type.equals("cd") && toHighlight[1] == true) {
      currentColor = color(10, 50, 70, 60);
    } else if(type.equals("dvd") && toHighlight[2] == true){
      currentColor = color(30, 50, 70, 60);
    } else if(type.equals("kit") && toHighlight[3] == true) {
      currentColor = color(50, 50, 70, 80); //kit and other are so rare that they get extra opacity
    } else if(type.equals("other") && toHighlight[4] == true) { 
      currentColor = color(1, 50, 70, 90);
    }
    if(bool == 1 && toHighlight[5] == true) {
      float currHue = hue(currentColor);
      float currSaturation = saturation(currentColor);
      float currBrightness = brightness(currentColor);
      float currAlpha = alpha(currentColor);
      currentColor = color(currHue, currSaturation*1.2, currBrightness*1.3, currAlpha+10);  
    } else if(bool == 0 && toHighlight[6] == true) {
      float currHue = hue(currentColor);
      float currSaturation = saturation(currentColor);
      float currBrightness = brightness(currentColor);
      float currAlpha = alpha(currentColor);
      currentColor = color(currHue, currSaturation*1.2, currBrightness*1.3, currAlpha+10);
    }
//  }
} 


// to draw text along an ellipse
// based on example 17-8 from http://www.learningprocessing.com
void drawArcText(String textString, float radius, float x, float y, float z, float rotateDegrees) {
  float arclength = 0;
  textAlign(CENTER);
  pushMatrix();
  translate(x,y,z);
  rotate(radians(rotateDegrees));
  for(int i=0; i<textString.length(); i++) {
    char currentChar = textString.charAt(i);
    float w = textWidth(currentChar); 
    arclength += w/2;
    float theta = PI + arclength / radius;
    pushMatrix();
    translate(radius*cos(theta), radius*sin(theta)); 
    rotate(theta + PI/2); 
    text(currentChar,0,0);
    popMatrix();
    arclength += w/2;
  } 
  popMatrix();
}


// to draw each line+arc segment
void drawLineArc(int firstDate, int secondDate, int currentLevel) {
  int first = firstDate;
  int second = secondDate;
  int level = currentLevel;
  float zCurrent = zStep*first/1.0; 
  float cyclePercent = (second-first)/360.0; //fraction of a circle
  pushMatrix();
  rotate(radians(first % 360 + 180));
  pushStyle();
  strokeWeight(2);    
  line(0,level*rIncrease,zCurrent, 0, (level+1)*rIncrease, zCurrent);
  popStyle();
  pushStyle();
  stroke(0, 0, 50, 20);
  strokeWeight(1);
  spiral(0,0,zCurrent, (level+1)*rIncrease, radians(4), 4*zStep, cyclePercent); //must multiply zStep by value inside "radians()"
  popStyle();
  popMatrix();
}
  

// spiral code based on a tutorial at http://faculty.purchase.edu/jeanine.meyer/processing/
void spiral(float x, float y, float z, float rad, float dr,float dh, float cycles) {
  float xxo,yyo,zzo;
  float xx,yy,zz;
  xxo=x-rad*cos(PI);
  yyo=y-rad*sin(PI);
  zzo=z;
//  zz = z+dh;
  zz = z-dh;
  pushMatrix();
  rotate(PI/2);
  for (float a=0;a<(2*PI*cycles);a+=dr) {
     xx = x+rad*cos(a);
     yy = y+rad*sin(a);
     zz+=dh;
     line(xxo,yyo,zzo,xx,yy,zz);  
     xxo=xx;
     yyo=yy;
     zzo=zz;
  }
  popMatrix();
}

// drawCylinder adapted from code at
// http://vormplus.be/blog/article/drawing-a-cylinder-with-processing
void drawCylinder(int sides, float r, float h) {
  float angle = 360 / sides;
  float halfHeight = h / 2;
    
  // draw top shape
  beginShape();
  for (int i = 0; i < sides; i++) {
    float x = cos( radians( i * angle ) ) * r;
    float y = sin( radians( i * angle ) ) * r;
    vertex( x, y, h );    
  }
  endShape(CLOSE);
    
  // draw bottom shape
  beginShape();
  for (int i = 0; i < sides; i++) {
    float x = cos( radians( i * angle ) ) * r;
    float y = sin( radians( i * angle ) ) * r;
    vertex( x, y, 0 );    
  }
  endShape(CLOSE);
    
  // draw body
  beginShape(TRIANGLE_STRIP);
  for (int i = 0; i < sides + 1; i++) {
    float x = cos( radians( i * angle ) ) * r;
    float y = sin( radians( i * angle ) ) * r;
    vertex( x, y, h);
    vertex( x, y, 0);    
  }
  endShape(CLOSE);
}

void setCamState() {
  if(frameRate < 10) {
    cam.setActive(false);
    cam.beginHUD();
    pushStyle();
    fill(70, 99, 50, 100);
    textFont(h16);
    textAlign(LEFT);
    text("Interactive mode disabled. Display fewer items to enable.", 20, 140);
    popStyle();
    cam.endHUD();
  } else if(mouseY < guiH) {
    cam.setActive(false);
  } else {
    cam.setActive(true); 
  }
//    println("frame rate: "+frameRate);
}


/* 
 * Event handlers
 */

public void handleSliderEvents(GValueControl slider, GEvent event) { 
  if(slider == sdr1) {
    rowsToDraw = sdr1.getValueI()+1;
  }
}

public void handleToggleControlEvents(GToggleControl option, GEvent event) {
  toHighlight = new boolean[7];
//  anyBoxChecked = false; 
// [ 0    1    2    3     4        5          6  ]
// {book, cd, dvd, kit, other, centralLoc, otherLoc}
  if (cbxTypeBk.isSelected()) {
    toHighlight[0] = true;
  } 
  if (cbxTypeCd.isSelected()) {
    toHighlight[1] = true;
  } 
  if (cbxTypeDvd.isSelected()) {
    toHighlight[2] = true;
  } 
  if (cbxTypeKit.isSelected()) {
    toHighlight[3] = true;
  } 
  if (cbxTypeOther.isSelected()) {
    toHighlight[4] = true;
  } 
  if (cbxLocCentral.isSelected()) {
    toHighlight[5] = true;
  } 
  if (cbxLocOther.isSelected()) {
    toHighlight[6] = true;
  } 
}

void keyPressed() {
  if (key == 's') {
    saveFrame("image.png");
  }
}
